# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
#
# Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
# copying or use of this file, via any medium, is strictly prohibited.
# Proprietary and confidential. You may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#     https://q-ctrl.com/terms
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS. See the
# License for the specific language.
"""Module for handling exceptions.
"""
from functools import wraps


class QctrlException(Exception):
    """Base class for all qctrl-related errors."""


def wrap_with_qctrl_exception(func):
    """Wraps any occuring exception with a `QctrlException`.
    The original exception can be accessed through `QctrlException.__cause__`

    Args:
      func: a callable

    Returns: the result of the callable

    """

    @wraps(func)
    def exception_wrapped_f(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as exc:
            raise QctrlException(f"{exc.__str__()}") from exc

    return exception_wrapped_f
