# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
#
# Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
# copying or use of this file, via any medium, is strictly prohibited.
# Proprietary and confidential. You may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#     https://q-ctrl.com/terms
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS. See the
# License for the specific language.
"""Utilities in common packages.
"""
import platform


def get_nested_value(dictionary: dict, ref: str, default=None):
    """
    Returns a nested value in dictionary
    of dictionaries. Returns default if
    cant be found. Example:

    >>> d = {"key1": {"key2": 1}}
    >>> get_nested_value(d, "key1.key2")
    1
    >>> get_nested_value(d, "key1.invalid")
    None
    >>> get_nested_value(d, "key1")
    {"key2": 1}
    """

    # unable to continue
    if not isinstance(dictionary, dict):
        return default

    try:
        key, remaining_ref = ref.split(".", 1)

    # no more nesting
    except ValueError:
        return dictionary.get(ref, default)

    # remaining nesting
    return get_nested_value(dictionary.get(key, default), remaining_ref, default)


def generate_user_agent(lib_name: str, version: str) -> str:
    """
    Generates custom user agents to be forward with API requests headers.
    """
    return (
        f"{lib_name}/{version} "
        f"({platform.platform()}) "
        f"Python/{platform.python_version()} "
        f"({platform.python_implementation()})"
    )
